/* screenmodel.h
 * Copyright (C) 2018 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301, USA.
**/
#ifndef SCREENMODEL_H
#define SCREENMODEL_H

#include <QAbstractListModel>
#include <QVector>
#include <QRect>

class QDesktopWidget;

class ScreenModel : public QAbstractListModel
{
    Q_OBJECT
public:
    ScreenModel(QObject *parent=0);
    int rowCount(const QModelIndex &parent = QModelIndex()) const;
    QVariant data(const QModelIndex &index, int role = Qt::UserRole) const;

private slots:
    void onScreenResized(int screen);
    void onScreenCountChanged(int newCount);
    void loadScreens();

private:
    QVector<QRect>  m_screen;
    QDesktopWidget *_desktop;
};

#endif // SCREENMODEL_H
